// ***************************************************************
// This file was created using the CreateProject.sh script
// for project CountingExp.
// CreateProject.sh is part of Bayesian Analysis Toolkit (BAT).
// BAT can be downloaded from http://www.mppmu.mpg.de/bat
// ***************************************************************

#include <BAT/BCLog.h>
#include <BAT/BCAux.h>
#include <BAT/BCDataSet.h>
#include <BAT/BCH1D.h>
#include <BAT/BCSummaryTool.h>

#include <TCanvas.h>
#include <TH1D.h>
#include <TLegend.h>

#include "CountingExp.h"

int main()
{
   // set nicer style for drawing than the ROOT default
   BCAux::SetStyle();

   // open log file
   BCLog::OpenLog("log.txt");
   BCLog::SetLogLevel(BCLog::detail);

   // create new CountingExp object
   CountingExp * m = new CountingExp();

   // create background measurement data point (T, N_1)=(100, 50)
   BCDataPoint * backgroundMeasurement = new BCDataPoint(2);
   backgroundMeasurement->SetValue(0, 100);
   backgroundMeasurement->SetValue(1, 50);

   //create background + signal  data point (T, N_2)=(100, 55)
   BCDataPoint* signalMeasurement = new BCDataPoint(2);
   signalMeasurement -> SetValue(0, 100);
   signalMeasurement -> SetValue(1, 55);

   // add the single single measurement to the data set
   BCDataSet * dataSet = new BCDataSet();
   dataSet->AddDataPoint(backgroundMeasurement);

   // register the data set with the model
   m->SetDataSet(dataSet);

   BCLog::OutSummary("Test model created");

   // create a new summary tool object
//   BCSummaryTool * summary = new BCSummaryTool(m);

   // perform your analysis here

   m->SetNbins("R_B", 300);
   m->SetNbins("R_S", 300);

   // normalize the posterior, i.e. integrate posterior
   // over the full parameter space
//   m->Normalize();

   // run MCMC and marginalize posterior wrt. all parameters
   // and all combinations of two parameters
   m->MarginalizeAll();

   // run mode finding; by default using Minuit
//   m->FindMode();

   // if MCMC was run before (MarginalizeAll()) it is
   // possible to use the mode found by MCMC as
   // starting point of Minuit minimization
   m->FindMode(m->GetBestFitParameters());

   // draw all marginalized distributions into a PostScript file
   m->PrintAllMarginalized("CountingExp_plots.ps");

   TCanvas * mycanvas = new TCanvas("mycanvas");
   m->GetMarginalized("R_S")->Draw(0, -95);
   mycanvas->Print("Limit.eps");

   // print all summary plots
//   summary->PrintParameterPlot("CountingExp_parameters.eps");
//   summary->PrintCorrelationPlot("CountingExp_correlation.eps");
//   summary->PrintKnowledgeUpdatePlots("CountingExp_update.ps");

   // calculate p-value
//   m->CalculatePValue( m->GetBestFitParameters() );

   // prepare a new plot
   mycanvas->Clear();

   // get marginal distribution P(R_B | N1 )
   TH1D * marginalHist1 = m->GetMarginalized("R_B")->GetHistogram();

   // create a copy of the histogram
   marginalHist1 = (TH1D*) marginalHist1->Clone("mHist1");

   // now fit again with N1 and N2
   dataSet->AddDataPoint(signalMeasurement);
   m->MarginalizeAll();
   m->FindMode(m->GetBestFitParameters());

   // get marginal distribution P(R_B|{N1,N2})
   TH1D * marginalHist2 = m->GetMarginalized("R_B")->GetHistogram();

   // draw second histogram in red
   marginalHist2->SetLineColor(2);

   // superimpose both distributions, normalize for easier comparison
   marginalHist2->DrawNormalized();
   marginalHist1->DrawNormalized("SAME");

   // add a legend to the plot
   TLegend * legend = new TLegend(0.7, 0.7, 0.85, 0.85);
   legend->AddEntry(marginalHist1, "N1", "l");
   legend->AddEntry(marginalHist2, "N1 and N2", "l");
   legend->Draw();

   // save a copy of the plot
   mycanvas->Print("R_B_update.eps");

   delete mycanvas;
   delete marginalHist1;
   delete legend;

   // print results of the analysis into a text file
   m->PrintResults("CountingExp_results.txt");

   delete dataSet;
   delete m;
//   delete summary;

   BCLog::OutSummary("Test program ran successfully");
   BCLog::OutSummary("Exiting");

   // close log file
   BCLog::CloseLog();

   return 0;
}

